package CompilerProj.Parser;

import CompilerProj.Lexical.*;
import CompilerProj.Semantic.*;

import static CompilerProj.Compiler.*;
import static CompilerProj.Lexical.TokenKind.*;
import static CompilerProj.Lexical.TokenSet.*;

import java.util.*;


/** This class represents the rewritten syntactic unit:
  **
  ** call-rest = ( [ argument-list ] )        */

public class CallRest extends SyntacticUnit {
    
    
    public static final TokenSet STARTS = ...;
    
    
    private Name     theName;    // name from NameStmt
    private AMethod  theMethod;  // the method
    private String   reg;        // register containing function result
    
        :
    
    public void parse ( ) {
        
        ArgumentList  anArgList;  // the argument list
        AVariable   qualifier;  // qualifier on call
        
        theMethod = asMethDcl(theName.getDecl());
            :
        qualifier = theName.getQualifier();
        if ( qualifier != null && qualifier.getType().equals(typeInFile) |
            qualifier.getType().equals(typeOutFile) ) {  // predefined I/O
            genIOCall(qualifier);
        }
        else {
            reg = allocReg();
            if ( qualifier != null && ! qualifier.isInstance() ) { // qualifier is local
                gen("var","0",String.valueOf(qualifier.getOffset()),reg);
            }
            else {       // qualifier is instance, this or super
                gen("var","0","-16",reg);
            };
            gen("val",reg);
            if ( qualifier != null && qualifier.isInstance() ) { // qualifier is instance
                gen("sel",String.valueOf(qualifier.getOffset()),reg);
                gen("val",reg);
            };        // reg -> object record
            expect(LPAREN,...);
            check(...);
            if ( tokenIn(ArgumentList.STARTS) ) {
                anArgList = new ArgumentList(...);
                anArgList.parse();
            }
            else {
                :
            };
            gen("psh",reg);   // push new this pointer
            expect(RPAREN,...);
            if ( theName.hasSuper() ) {
                gen("cst",((Classes)scope.getSuperclass()).getIdentifier(),reg); // reg -> superclass class record
            }
            else {
                gen("val",reg);   // reg -> class record
            }
            gen("sel",String.valueOf(theMethod.getOffset()),reg);
            gen("val",reg);   // reg -> method code
            gen("cal","0",reg);  // call method
            freeReg(reg);
            if ( theMethod.isFunction() ) {
                reg = "rr";  // function result in result register
            }
            else {
                reg = null;
            };
        };
        
    }; // parse
    
    
    private void genIOCall ( AVariable qualifier ) {...}; // genIOCall
    
    
} // CallRest