package CompilerProj.Semantic;

import static CompilerProj.Compiler.*;

import java.util.*;


/** This class represents the declaration of a class. A class has a superclass,
  ** a constructor, and a set of members that are declared within the class.  */


public class Classes extends Types implements AClass, AType, Scope {
    
    
    protected AClass  superclass;  // the superclass
        :
    protected int     varOffset;   // offset to next variable
    protected int     methOffset;  // offset to next method
    
    
    public Classes ( String id, AClass s ) {
        
            :
        superclass = s;
            :
        if ( superclass == null ) {
            varOffset = 1;
            methOffset = 2;
        }
        else {      // extending superclass
            varOffset = superclass.getObjectSize();
            methOffset = superclass.getClassSize();
        };
        
    }; // constructor
    
        :
    
    public void fillMethodTable ( String table[] ) {
        
        Iterator  it;
        ADeclaration dcl;
        
        if ( superclass != null ) {
            superclass.fillMethodTable(table);
        };
        table[theConstructor.getOffset()] = identifier + "_" + theConstructor.getIdentifier();
        it = members.values().iterator();
        while ( it.hasNext() ) {
            dcl = (ADeclaration)it.next();
            if ( dcl instanceof AMethod ) {
                table[((AMethod)dcl).getOffset()] = identifier + "_" + dcl.getIdentifier();
            };
        };
        
    }; // fillMethodTable
    
        :
    
    public void define ( ADeclaration dcl ) {
        
        AVariable    var;
        AMethod      meth;
        ADeclaration superDcl;  // declaration of name in superclass
        
        if      ( dcl instanceof AVariable ) {
            var = (AVariable)dcl;
            var.setOffset(varOffset);
            var.setInstance(true);
            varOffset = varOffset + var.getType().getLength();
        }
        else if ( dcl instanceof AMethod ) {
            meth = (AMethod)dcl;
            if ( superclass != null ) {
                superDcl = superclass.find(meth.getIdentifier());
                if ( superDcl != null && superDcl instanceof AMethod ) {  // overriding method
                    if ( ((AMethod)superDcl).sigMatch(meth) ) {           // check signatures
                        meth.setOffset(((AMethod)superDcl).getOffset());
                    }
                    else {
                        listing.writeError(...);
                    }
                }
                else {
                    meth.setOffset(methOffset);
                    methOffset = methOffset + 1;
                };
            }
            else {
                meth.setOffset(methOffset);
                methOffset = methOffset + 1;
            };
        };
        members.put(dcl.getIdentifier(),dcl);
        
    }; // define
    
        :
            
} // Classes
