package CompilerProj.Parser;

import CompilerProj.Lexical.*;
import CompilerProj.Semantic.*;

import static CompilerProj.Compiler.*;
import static CompilerProj.Lexical.TokenKind.*;
import static CompilerProj.Lexical.TokenSet.*;


/** This class represents the added syntactic unit:
  **
  ** if-stmt = if expression then { statement } [ else { statement } ] end ; */

public class IfStmt extends SyntacticUnit {
    
    
    public static final TokenSet STARTS = ...;
    
    
    public IfStmt ( Scope c, TokenSet s ) {
        
        super(c,s);
        
    }; // constructor
    
    
    public void parse ( ) {
        
        Expression anExpression;  // the condition
        Statement  aStmt;         // a statement
        String     reg;           // register for expression
        String     endLab;        // label for else/end
        String     tmpLab;        // label
        
        expect(IF,...);
        anExpression = new Expression(...);
        anExpression.parse();
            :
        expect(THEN,...);
        reg = anExpression.getReg();
        endLab = newLab();
        gen("brf",reg,endLab);
        freeReg(reg);
        while ( true ) {
            check(...);
        if ( tokenIn(oneOf(stopSet,ELSE,END,SEMI)....) ) break;
            if ( tokenIn(Statement.STARTS) ) {
                aStmt = new Statement(...);
                aStmt.parse();
            };
        };
        check(...);
        if ( tokenIs(ELSE) ) {
                :
            tmpLab = newLab();
            gen("bra",tmpLab);
            genLab(endLab);
            endLab = tmpLab;
            while ( true ) {
                check(...);
                if ( tokenIn(oneOf(stopSet,END,SEMI)...) ) break;
                if ( tokenIn(Statement.STARTS) ) {
                    aStmt = new Statement(...);
                    aStmt.parse();
                };
            };
        };
        expect(END,...);
        expect(SEMI,...);
        genLab(endLab);
        
    }; // parse
    
    
} // IfStmt