package CompilerProj.Parser;

import CompilerProj.Lexical.*;
import CompilerProj.Semantic.*;

import static CompilerProj.Compiler.*;
import static CompilerProj.Lexical.TokenKind.*;
import static CompilerProj.Lexical.TokenSet.*;


/** This class represents a generic syntactic unit. It provides standard methods
  ** in support of parsing, semantic analysis and code generation. Every syntactic
  ** unit has a scope and a set of stop tokens for parsing.      */

public abstract class SyntacticUnit {
    
        :    
    
    public abstract void parse ( ) ;
    
        :
    
// Code Generation helper methods
    
    /** This method returns an unused register if available otherwise
      * generates a compiler error.
      * 
      * @return  String  name of unused register                        */
            
    protected String allocReg ( ) {...}; // allocReg
    
    
    /** This method marks the register as available. If already available
      * it generates a compiler error.
      * 
      * @param  reg  name of register to be freed                       */
            
    protected void freeReg ( String reg ) {...}; // freeReg
    
    
    /** This method generates code for a variable reference.
      *
      * @param  qualifier  qualifier variable (null for none or super)
      * @param  var        variable being referenced                    */
            
    protected String varReference ( AVariable qualifier, AVariable var ) {...}; // varReference
    
    
    /** This method returns a new label qualified by the class name.
      * 
      * @return  String  new label                                      */
            
    protected String newLab ( ) {...}; // newLab
    
    
    /** This method emits the label on the next line of code generated
      * by gen.
      * 
      * @param  lab  a label for the next line of code                  */
            
    protected void genLab ( String lab ) {...}; // emitLab
    
    
    /** This method emits a lione of code to the code file. The line includes
      * the label(s) emitted by genLab since the last call to gen, followed
      * by the opCode and then 0 or more operands separated by commas.
      *
      * @param  opcode  opcode for instruction
      * @param  opnds   0 or more operands                              */
            
    protected void gen ( String opcode, String... opnds ) {...}; // gen
    
    
} // SyntacticUnit